/* futebus example: poll() based non-blocking reader on broadcast channel

   License: CC0 - no rights reserved

   How to use:
   1. create the bus: futebus bus
   2. run the master: ./Bpoll
   3. observe that Bpoll is idling, reporting 0 readable after poll calls
   4. run a process to feed in a message: echo hello >>bus/B1
   5. observe that Bpoll reports readable after a poll call and reads the bus
*/

#include <stdio.h>
#include <poll.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>

int main(int argc, char* argv[])
{
	int fd, r;
	struct pollfd pfd;

	/* open the channel */
	fd = open("bus/B1", O_APPEND|O_RDWR);
	if (fd < 0) {
		perror("open()");
		exit(1);
	}

	/* set up for poll: the channel is always writable, do only POLLIN */
	pfd.fd = fd;
	pfd.events = POLLIN;

	while((r=poll(&pfd, 1, 1000)) >= 0) {
		printf("poll() returned: %i, revents=%i\n", r, pfd.revents);

		if (pfd.revents & POLLIN) {
			char buf[512];
			int n = read(fd, buf, sizeof(buf));

			if(n > 0) {
				buf[n] = 0;
				fputs(buf, stdout);
			}
		}
	}

	perror("poll()");
	exit(1);
}
